﻿using OpcLabs.EasyOpc;
using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.EasyOpc.DataAccess.OperationModel;

namespace QuickOpcExample
{
    public class Program
    {
        private static readonly DADataSource _dataSource = DADataSource.ByValueAge;
        private static readonly int _valueAge = 1000;
        private static readonly int _collectionRate = 500;
        private static readonly float _percentDeadband = 0;
        private static readonly ServerDescriptor _serverDescriptor = new("Matrikon.OPC.Simulation.1");
        private static readonly string _itemId = ".A";

        public static async Task Main(string[] args)
        {
            var opcClient = new EasyDAClient();
            opcClient.InstanceParameters.Mode.AllowAsynchronousMethod = false;

            var itemSubscriptionArguments = new List<EasyDAItemSubscriptionArguments>
            {
                new EasyDAItemSubscriptionArguments(
                    new DAItemGroupArguments(
                        _serverDescriptor,
                        new DAItemDescriptor(_itemId),
                        new DAGroupParameters(_collectionRate, _percentDeadband),
                        "this is the subscription item state")),
            };

            opcClient.ItemChanged += (sender, eventArgs) =>
            {
                // Simulate some work which is done in some other part of application, but almost immediately after item update, in parallel
                // Note: this is not how it is done in the real application, but it is a simplified version that demonstrates the issue
                Task.Run(async () =>
                {
                    var readItemArguments = new List<DAReadItemArguments>
                    {
                        new DAReadItemArguments(_serverDescriptor, _itemId, new DAReadParameters(_dataSource, _valueAge), "this is the desired state"),
                    };

                    DAVtqResult[] readResults = opcClient.ReadMultipleItems(readItemArguments.ToArray());

                    // Issue is here, we expect to see "this is the desired state"
                    Console.WriteLine(readResults[0].State?.GetType());
                });
            };

            opcClient.SubscribeMultipleItems(itemSubscriptionArguments.ToArray());
            await Task.Delay(5000, CancellationToken.None);
        }
    }
}